/*
 *
 * Copyright (C) 2000  Robert Olsson, Jens Ls 
 * Swedish University of Agricultural Sciences
 *
 * This file is part of GNU Zebra.
 *
 * GNU Zebra is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * GNU Zebra is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GNU Zebra; see the file COPYING.  If not, write to the Free
 * Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.  
 */

#include <zebra.h>
#include "vector.h"
#include "vty.h"
#include "command.h"
#include "getopt.h"
#include "thread.h"

#include "stream.h"
#include "network.h"
#include "prefix.h"
#include "version.h"
#include "memory.h"
#include "command.h"

#include "if.h"
#include "sockunion.h"

#include "log.h"

#include "linklist.h"

#define TESTER_VTY_PORT 2699
#define TESTER_VTYSH_PATH     "/tmp/zebra_tester"
#define PATH_TESTER_PID "/var/run/zebra_tester.pid"

extern struct host host;

void tester_init();

/*  options, we use GNU getopt library. */
struct option longopts[] = 
{
  { "daemon",      no_argument,       NULL, 'd'},
  { "lists",       no_argument,       NULL, 'l'},
  { "vty_port",    required_argument, NULL, 'P'},
  { "version",     no_argument,       NULL, 'v'},
  { "help",        no_argument,       NULL, 'h'},
  { 0 }
};

/* tester program name. */
char *progname;

/* Master of threads. */
struct thread_master *master;

/* Lists */
list test_list;

int vty_port = TESTER_VTY_PORT;

static void
usage (int status)
{
  if (status != 0)
    fprintf (stderr, "Try `%s --help' for more information.\n", progname);
  else
    {    
      printf ("Usage : %s [OPTION...]\n\n\
-d, --daemon       Runs in daemon mode\n\
-l, --lists        Test lists\n\
-P, --vty_port     Set vty's port number\n\
-v, --version      Print program version\n\
-h, --help         Display this help and exit\n\
\n\
Report bugs to %s\n", progname, ZEBRA_BUG_ADDRESS);
    }

  exit (status);
}

/* SIGHUP handler. */
void 
sighup (int sig)
{
  zlog (NULL, LOG_INFO, "SIGHUP received");

  /* Terminate all thread. */
  zlog_info ("tester restarting!");

  /* Create VTY's socket */
  vty_serv_sock (vty_port ? vty_port : TESTER_VTY_PORT, TESTER_VTYSH_PATH); 
  
  /* Try to return to normal operation. */
}

/* SIGINT handler. */
void
sigint (int sig)
{
  zlog (NULL, LOG_INFO, "Terminating on signal");

  exit (0);
}

/* SIGUSR1 handler. */
void
sigusr1 (int sig)
{
  zlog_rotate (NULL);
}

/* Signale wrapper. */
/* RETSIGTYPE * */
void
signal_set (int signo, void (*func)(int))
{
  int ret;
  struct sigaction sig;
  struct sigaction osig;

  sig.sa_handler = func;
  sigemptyset (&sig.sa_mask);
  sig.sa_flags = 0;
#ifdef SA_RESTART
  sig.sa_flags |= SA_RESTART;
#endif /* SA_RESTART */

  ret = sigaction (signo, &sig, &osig);

  /*  if (ret < 0) 
    return (SIG_ERR);
  else
    return (osig.sa_handler);*/
}

/* Initialization of signal handles. */
void
signal_init ()
{
  signal_set (SIGHUP, sighup);
  signal_set (SIGINT, sigint);
  signal_set (SIGTERM, sigint);
  signal_set (SIGPIPE, SIG_IGN);
  signal_set (SIGUSR1, sigusr1);
}

/* Main routine of tester. Treatment of argument
   is handled at here. */

int
main (int argc, char **argv)
{
  char *p;
  int opt;
  int daemon_mode = 0;

  struct thread thread;

  /* Preserve name of myself. */
  progname = ((p = strrchr (argv[0], '/')) ? ++p : argv[0]);
  
  zlog_default = openzlog (progname, ZLOG_NOLOG, ZLOG_PIM,
                           LOG_CONS|LOG_NDELAY|LOG_PID, LOG_DAEMON);

  /* Command line argument treatment. */
  while (1) 
    {
      opt = getopt_long (argc, argv, "dlhP:v", longopts, 0);
    
      if (opt == EOF)
        break;

      switch (opt) 
        {
        case 0:
          break;
        case 'd':
          daemon_mode = 1;
          break;
        case 'P':
          vty_port = atoi (optarg);
          break;
        case 'v':
          print_version ();
          exit (0);
          break;
        case 'h':
          usage (0);
          break;
        default:
          usage (1);
          break;
        }
    }

 /* Make thread master. */
  master = thread_make_master ();

  /* Initializations. */
  signal_init ();
  cmd_init();
  vty_init();
  memory_init();

  tester_init();

  host.password = "test";

/* Change to the daemon program. */
  if (daemon_mode)
    daemon (0, 0);

  /* Pid file create. */
  pid_output (PATH_TESTER_PID);

  /* Create pim server */
  vty_serv_sock (vty_port ? vty_port : TESTER_VTY_PORT, TESTER_VTYSH_PATH);
 
  /* Execute each thread. */
  while (thread_fetch (master, &thread))
    thread_call (&thread);
    
  /* Not reached. */
  exit (0);
}

#define TEST_LIST_SIZE 1024
#define TEST_LIST_ITERATIONS 256

DEFUN (test,
       test_cmd,
       "test TYPE",
       "perform test\n"
       "perform test\n")
{
  int i, iter;
  
  if(!strcmp("lists", argv[0] ))
    {
      vty_out(vty, "Performing test on %s%s", 
	      argv[0], 
	      VTY_NEWLINE);
            
      for(iter=0; iter < TEST_LIST_ITERATIONS; iter++)
	{
	  test_list = list_init();
	  for(i=0; i< TEST_LIST_SIZE; i++)
	    list_add_node(test_list, (void *) i);
	  
	  list_free(test_list);
	}
      
    }
  
  return CMD_SUCCESS;
}

DEFUN (show_tests,
       show_tests_cmd,
       "show tests",
       "Show available tests\n"
       "Show available tests\n")
{
  vty_out(vty, "\tAvailable tests%s", VTY_NEWLINE);
  vty_out(vty, "lists%s", VTY_NEWLINE);
  
  return CMD_SUCCESS;
}

void
tester_init()
{

  /* Install top nodes. */

   install_element (VIEW_NODE, &show_tests_cmd);
   install_element (VIEW_NODE, &test_cmd);

}

/*

gcc -Wall -g -I../lib -c lib_test.c;gcc -o tester lib_test.o ../lib/libzebra.a 

*/


